/**
 * Copyright 2007-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.maven.jbuilding;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.Map;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.repository.layout.ArtifactRepositoryLayout;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugins.annotations.Component;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.codehaus.plexus.util.FileUtils;

/**
 * Get the declared files and copy them in the given directory.
 * @since 2.0.0
 */
@Mojo(name = "assemble",
      defaultPhase = LifecyclePhase.PROCESS_SOURCES)
public class AssembleArtifactItemsMojo extends AbstractResolverMojo {

    /**
     * Location new resources directory.
     * @since 2.0.0
     */
    @Parameter(defaultValue = "${project.build.directory}/assemble-resources",
               required = true)
    private File output;

    /**
     * Where do I place my artifacts ?
     * @since 2.0.0
     */
    @Parameter(defaultValue = "repositories/maven2-internal",
               required = true)
    private String directory;

    /**
     * Allow timestamp in SNAPSHOT version if set to true
     * @since 2.0.0
     */
    @Parameter(defaultValue = "false")
    private boolean allowTimestamp = false;

    @Component(hint = "default")
    private ArtifactRepositoryLayout layout;

    /**
     * @throws MojoExecutionException When I cannot copy the artifacts in the
     *         output directory.
     * @see org.ow2.util.maven.jbuilding.AbstractResolverMojo#execute()
     */
    public void execute() throws MojoExecutionException {

        Map<String, Collection<Artifact>> map = resolveArtifacts(
                SERVER_SIDE,
                BOTH_MODE,
                ALL_ARTIFACT_ITEMS,
                this.allowTimestamp);

        // OK, now we're sure that everything is in the local repository
        // We have to copy theses files in the output directory
        File repositoryDirectory = new File(output, directory.replace('/', File.separatorChar));

        // Create the directory
        if (!repositoryDirectory.exists()) {
            if (!repositoryDirectory.mkdirs()) {
                throw new MojoExecutionException("Cannot create directory: " + repositoryDirectory);
            }
        }

        // List all plans
        for (Collection<Artifact> artifacts: map.values()) {
            // Copy the artifacts in the directory
            for (Artifact artifact : artifacts) {

                String path = layout.pathOf( artifact );
                if (!this.allowTimestamp && !artifact.getVersion().equals(artifact.getBaseVersion())) {
                    path = path.replace(artifact.getVersion(), artifact.getBaseVersion());
                }

                final File source = artifact.getFile();
                final File target = new File(repositoryDirectory, path);
                
                try {
                    FileUtils.copyFileIfModified(source, target);
                } catch (IOException e) {
                    throw new MojoExecutionException("Cannot copy bundle " + artifact + "into " + repositoryDirectory, e);
                }
            }
        }

        project.addCompileSourceRoot(output.getPath());

    }

}
