/**
 * Copyright 2007-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.maven.jbuilding;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.model.Resource;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.codehaus.plexus.util.FileUtils;

/**
 * Get the declared files and copy them in the given directory.
 * @since 2.0.0
 */
@Mojo(name = "generate-list")
public class GenerateBundleListMojo extends AbstractResolverMojo {

    /**
     * Location new resources directory.
     * @since 2.0.0
     */
    @Parameter(defaultValue = "${project.build.directory}/generated-resources")
    private File output;

    /**
     * Where do I place my file ?
     * @since 2.0.0
     */
    @Parameter(defaultValue = "org/ow2/jonas/launcher/felix/jonas-autodeploy-bundles.properties")
    private String location;

    /**
     * Allow timestamp in SNAPSHOT version if set to true
     * @since 2.0.0
     */
    @Parameter(defaultValue = "false")
    private boolean allowTimestamp = false;

    /**
     * @throws MojoExecutionException When I cannot copy the artifacts in the
     *         output directory.
     * @see org.ow2.util.maven.jbuilding.AbstractResolverMojo#execute()
     */
    public void execute() throws MojoExecutionException {
        Collection<Artifact> artifactsToInstall = resolveArtifacts(SERVER_SIDE, INSTALL_MODE, AUTO_DEPLOY_ARTIFACT_ITEMS,
                this.allowTimestamp).get(AUTO_DEPLOY_ARTIFACT_ITEMS);
        Collection<Artifact> artifactsToStart = resolveArtifacts(SERVER_SIDE, START_MODE, AUTO_DEPLOY_ARTIFACT_ITEMS,
                this.allowTimestamp).get(AUTO_DEPLOY_ARTIFACT_ITEMS);

        // OK, now we're sure that everything is in the local repository
        // We will generate a property file
        File propertyFile = new File(output, location.replace('/', File.separatorChar));

        File parent = propertyFile.getParentFile();
        // Create the directory
        if (!parent.exists()) {
            if (!parent.mkdirs()) {
                throw new MojoExecutionException("Cannot create directory: " + parent);
            }
        }

        // Generate the property
        StringBuilder sb = new StringBuilder();
        sb.append("# Generated " + new Date() + " by JOnAS.");

        if (artifactsToInstall != null) {
            Map<Integer, Collection<Artifact>> startLevels = groupByLevel(artifactsToInstall);
            for (Integer startLevel: startLevels.keySet()) {
                sb.append("\n\n");
                sb.append("install.level." + startLevel + " ");
                sb.append(generateArtifacts(startLevels.get(startLevel)));
            }
        }
        if (artifactsToStart != null) {
            Map<Integer, Collection<Artifact>> startLevels = groupByLevel(artifactsToStart);
            for (Integer startLevel: startLevels.keySet()) {
                sb.append("\n\n");
                sb.append("start.level." + startLevel + " ");
                sb.append(generateArtifacts(startLevels.get(startLevel)));
            }
        }

        // add a final EOL
        sb.append("\n");

        // Flush the content
        try {
            FileUtils.fileWrite(propertyFile.getPath(), sb.toString());
        } catch (IOException e) {
            throw new MojoExecutionException("Cannot create file " + propertyFile);
        }

        // Add the generated file in the resources of maven
        Resource resource = new Resource();
        resource.setDirectory(output.getPath());
        project.addResource(resource);
    }

    private Map<Integer, Collection<Artifact>> groupByLevel(final Collection<Artifact> artifacts) {
        Map<Integer, Collection<Artifact>> startLevels = new HashMap<Integer, Collection<Artifact>>();

        for (Artifact artifact: artifacts) {
            Integer startLevel = artifactItemMap.get(artifact.getDependencyConflictId()).getStartlevel();
            Collection<Artifact> startLevelGroup = startLevels.get(startLevel);
            if (startLevelGroup == null) {
                startLevelGroup = new LinkedList<Artifact>();
            }
            startLevelGroup.add(artifact);
            startLevels.put(startLevel, startLevelGroup);
        }
        return startLevels;
    }

    private String generateArtifacts(final Collection<Artifact> artifacts) {
        StringBuilder sb = new StringBuilder();
        boolean first = true;

        for (Artifact artifact : artifacts) {
            // Add a comma and go to the line
            if (!first) {
                sb.append(", \\");
                sb.append("\n");
                sb.append("              ");
            }
            String groupId = artifact.getGroupId();
            String artifactId = artifact.getArtifactId();
            String version = artifact.getVersion();
            if (artifact.isSnapshot() && !this.allowTimestamp) {
                version = artifact.getBaseVersion();
            }
            String classifier = artifact.getClassifier();

            sb.append(groupId);
            sb.append(":");
            sb.append(artifactId);
            sb.append(":");
            sb.append(version);
            if (classifier != null) {
                sb.append(":{");
                sb.append(classifier);
                sb.append("}");
            }
            first = false;
        }

        return sb.toString();
    }

}
