/**
 * Copyright 2007-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.maven.jbuilding;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.Date;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.model.Resource;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.codehaus.plexus.util.FileUtils;

/**
 * Generate maven2 deployment plans describe in the plugin configuration.
 * @author Francois Fornaciari
 * @since 2.0.0
 */
@Mojo(name = "generate-maven2-deployment-plans")
public class GenerateMaven2DeploymentPlansMojo extends AbstractResolverMojo {

    /**
     * Location new resources directory.
     * @since 2.0.0
     */
    @Parameter(defaultValue = "${project.build.directory}/configuration-resources")
    private File output;

    /**
     * Location of the generated maven2 deployment plans
     * @since 2.0.0
     */
    @Parameter(defaultValue = "repositories/url-internal")
    private String directory;

    /**
     * Allow timestamp in SNAPSHOT version if set to true
     * @since 2.0.0
     */
    @Parameter(defaultValue = "false")
    private boolean allowTimestamp = false;

    /**
     * @throws MojoExecutionException When I cannot copy the artifacts in the
     *         output directory.
     * @see org.ow2.util.maven.jbuilding.AbstractResolverMojo#execute()
     */
    public void execute() throws MojoExecutionException {
        Map<String, Collection<Artifact>> plans = resolveArtifacts(SERVER_SIDE, BOTH_MODE, DEPLOYMENT_PLANS_ARTIFACT_ITEMS,
                this.allowTimestamp);

        // Generate maven2 deployment plans describe in the plugin configuration.
        Set<Entry<String, Collection<Artifact>>> deploymentPlanEntries = plans.entrySet();
        for (Entry<String, Collection<Artifact>> deploymentPlanEntry : deploymentPlanEntries) {
            String deploymentPlanName = deploymentPlanEntry.getKey();
            File file = new File(getDeploymentPlanDirectory(deploymentPlanName), deploymentPlanName.concat(".xml"));
            String[] includes = deploymentPlanIncludes.get(deploymentPlanName);
            generateDeploymentPlan(file, deploymentPlanEntry.getValue(), includes, deploymentPlanName);
        }
    }

    /**
     * @param file
     * @param artifacts
     * @param includes
     * @param deploymentPlanName the deployment plan name
     * @throws MojoExecutionException
     */
    private void generateDeploymentPlan(final File file, final Collection<Artifact> artifacts,
            final String[] includes, final String deploymentPlanName)
            throws MojoExecutionException {
        StringBuilder sb = new StringBuilder();
        sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
        sb.append("<!-- Generated " + new Date() + " by JOnAS. -->\n");
        sb.append("<deployment-plan xmlns=\"http://jonas.ow2.org/ns/deployment-plan/1.0\"\n"
                + "  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
                + "  xmlns:m2=\"http://jonas.ow2.org/ns/deployment-plan/maven2/1.0\"\n"
                + "  xmlns:url=\"http://jonas.ow2.org/ns/deployment-plan/url/1.0\"\n"
                + "  xsi:schemaLocation=\"http://jonas.ow2.org/ns/deployment-plan/1.0 http://jonas.ow2.org/ns/deployment-plan/1.0/deployment-plan-1.0.xsd\n"
                + "                      http://jonas.ow2.org/ns/deployment-plan/maven2/1.0 http://jonas.ow2.org/ns/deployment-plan/maven2-deployment-plan-1.0.xsd\"\n"
                + "  atomic=\"false\" id=\"" + deploymentPlanName + "\">\n");

        if (includes != null) {
            for (String include : includes) {
                sb.append("  <deployment id=\"" + include + "\" xsi:type=\"url:url-deploymentType\">\n");
                sb.append("    <url:resource>" + include + ".xml</url:resource>\n");
                sb.append("  </deployment>\n");
            }
        }

        for (Artifact artifact : artifacts) {
            // Get the reloadable mode
            boolean reloadable = artifactItemMap.get(artifact.getDependencyConflictId()).isReloadable();

            // Get the bundle start mode
            boolean start = artifactItemMap.get(artifact.getDependencyConflictId()).isStart();

            // Get the bundle reference mode
            boolean reference = artifactItemMap.get(artifact.getDependencyConflictId()).isReference();

            // Get the bundle start level
            int startLevel = artifactItemMap.get(artifact.getDependencyConflictId()).getStartlevel();

            // Get the bundle start operation mode
            boolean startTransient = artifactItemMap.get(artifact.getDependencyConflictId()).isStartTransient();

            sb.append("  <deployment id=\"" + artifact.getDependencyConflictId()
                    + "\" xsi:type=\"m2:maven2-deploymentType\" reloadable=\"" + reloadable + "\" start=\"" + start
                    + "\" reference=\"" + reference + "\" startlevel=\"" + startLevel + "\" starttransient=\"" + startTransient
                    + "\">\n");
            sb.append("    <m2:groupId>" + artifact.getGroupId() + "</m2:groupId>\n");
            sb.append("    <m2:artifactId>" + artifact.getArtifactId() + "</m2:artifactId>\n");

            if (artifact.getClassifier() != null) {
                sb.append("    <m2:classifier>" + artifact.getClassifier() + "</m2:classifier>\n");
            }

            String version;
            if (this.allowTimestamp) {
                version = artifact.getVersion();
            } else {
                version = artifact.getBaseVersion();
            }
            sb.append("    <m2:version>" + version + "</m2:version>\n");
            sb.append("    <m2:type>" + artifact.getType() + "</m2:type>\n");

            sb.append("  </deployment>\n");
        }

        sb.append("</deployment-plan>");

        // add a final EOL
        sb.append("\n");

        // Flush the content
        try {
            FileUtils.fileWrite(file.getPath(), sb.toString());
        } catch (IOException e) {
            throw new MojoExecutionException("Cannot create file " + file);
        }

        // Add the generated file in the resources of maven
        Resource resource = new Resource();
        resource.setDirectory(output.getPath());
        project.addResource(resource);
    }

    /**
     * @return the output file.
     */
    public File getOutput() {
        return output;
    }

    /**
     * @return the directory.
     */
    public String getDirectory() {
        return directory;
    }

    /**
     * Set the given output.
     * @param output the output file
     */
    public void setOutput(final File output) {
        this.output = output;
    }

    /**
     * Sets the given directory.
     * @param directory the directory path
     */
    public void setDirectory(final String directory) {
        this.directory = directory;
    }

    private File getDeploymentPlanDirectory(final String deploymentPlanName) throws MojoExecutionException {
        String planDirectoryLocation = directory;

        DeploymentPlan deploymentPlan = deploymentPlanMap.get(deploymentPlanName);
        if (deploymentPlan != null && deploymentPlan.getDirectory() != null) {
            planDirectoryLocation = deploymentPlan.getDirectory();
        }


        File planDirectoryLocationFile = new File(output, planDirectoryLocation.replace('/', File.separatorChar));
        // Create the directory
        if (!planDirectoryLocationFile.exists()) {
            if (!planDirectoryLocationFile.mkdirs()) {
                throw new MojoExecutionException("Cannot create directory: " + planDirectoryLocationFile);
            }
        }

        return planDirectoryLocationFile;
    }
}
