/**
 * Copyright 2007-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.maven.jbuilding;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.regex.Pattern;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.DefaultArtifact;
import org.apache.maven.artifact.handler.DefaultArtifactHandler;
import org.apache.maven.artifact.resolver.ArtifactResolutionRequest;
import org.apache.maven.artifact.resolver.ArtifactResolutionResult;
import org.apache.maven.artifact.resolver.ResolutionNode;
import org.apache.maven.plugin.testing.AbstractMojoTestCase;
import org.apache.maven.plugin.testing.stubs.ArtifactStub;
import org.apache.maven.repository.RepositorySystem;
import org.codehaus.plexus.util.FileUtils;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.ow2.util.maven.jbuilding.stubs.TestableMavenProject;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.when;

/**
 * The VersionsMojoTestCase is ...
 *
 * @author Guillaume Sauthier
 */
public class VersionsMojoTestCase extends AbstractMojoTestCase {

    @Mock
    private RepositorySystem repositorySystem;

    private DefaultArtifact mavenCoreArtifact = new DefaultArtifact("org.apache.maven", "maven-core", "2.0", null, "jar", null, new DefaultArtifactHandler("jar"));

    @Mock
    private ArtifactResolutionResult mavenCoreResult;

    @Override
    protected void setUp() throws Exception {
        // required for mojo lookups to work
        super.setUp();
        MockitoAnnotations.initMocks(this);
    }

    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testMojoFileCreation() throws Exception {

        VersionsMojo mojo = lookupConfiguredVersionsMojo();

        // Safety
        String filename = getBasedir() + File.separator + "target" + File.separator + "versions.properties";
        FileUtils.fileDelete(filename);
        assertFalse("versions.properties file was not removed", FileUtils.fileExists(filename));

        mojo.execute();

        assertTrue("versions.properties file was not written", FileUtils.fileExists(filename));

        // Ensure that the properties file is valid
        Properties loadable = new Properties();
        loadable.load(new FileInputStream(filename));

        assertEquals("Only 2 properties should be included", 2, loadable.size());
        assertEquals("1.0.0", loadable.getProperty("included.version"));
        assertEquals("2.0", loadable.getProperty("org.apache.maven_maven-core"));
    }

    public void testMojoExecutionOnModelProperties() throws Exception {

        VersionsMojo mojo = lookupVersionsMojo();

        Map<String, String> props = new HashMap<String, String>();
        mojo.getVersionsFromModelProperties(props);

        assertEquals("Only 1 property should be included", 1, props.size());
        assertEquals("Property value is wrong", "1.0.0", props.get("included.version"));
    }

    public void testMojoExecutionOnProjectDependencies() throws Exception {

        VersionsMojo mojo = lookupVersionsMojo();

        Map<String, String> props = new HashMap<String, String>();
        mojo.getVersionsFromProjectDependencies(props);

        assertEquals("Only 1 property should be included", 1, props.size());
        assertEquals("Property value is wrong", "2.0", props.get("org.apache.maven_maven-core"));
    }

    public void testMojoExecutionOnBundleItems() throws Exception {

        VersionsMojo mojo = lookupVersionsMojo();

        Map<String, String> props = new HashMap<String, String>();

        // Prepare a pseudo resolved map of artifacts
        Artifact artifact = createArtifact("org.ow2.util.test", "test-module", "3.0");
        Collection<Artifact> artifacts = Collections.singleton(artifact);
        Map<String, Collection<Artifact>> map = new HashMap<String, Collection<Artifact>>();
        map.put("not used key", artifacts);

        mojo.getVersionsFromArtifactItems(map, props);

        assertEquals("Only 1 property should be included", 1, props.size());
        assertEquals("Property value is wrong", "3.0", props.get("org.ow2.util.test_test-module"));
    }

    public void testMojoExecutionNameSimplification() throws Exception {

        VersionsMojo mojo = lookupVersionsMojo();

        Map<String, String> props = new HashMap<String, String>();

        // Prepare a pseudo resolved map of artifacts
        Artifact artifact = createArtifact("org.ow2.util.test", "org.ow2.util.test.test-module", "3.0");
        Collection<Artifact> artifacts = Collections.singleton(artifact);
        Map<String, Collection<Artifact>> map = new HashMap<String, Collection<Artifact>>();
        map.put("not used key", artifacts);

        mojo.getVersionsFromArtifactItems(map, props);

        assertEquals("Only 1 property should be included", 1, props.size());
        assertEquals("Property value is wrong", "3.0", props.get("org.ow2.util.test_test-module"));
    }

    public void testMojoExecutionFiltering() throws Exception {

        VersionsMojo mojo = lookupVersionsMojo();

        Map<String, String> props = new HashMap<String, String>();

        // Prepare a pseudo resolved map of artifacts
        Artifact a1 = createArtifact("org.ow2.util.test.filter", "test-module", "3.0");
        Artifact a2 = createArtifact("org.ow2.util.test.filter", "test-filtered-module", "3.0");
        Artifact a3 = createArtifact("org.ow2.util.test.filter", "test-unmatched-version", "3.1");

        Collection<Artifact> artifacts = new ArrayList<Artifact>();
        artifacts.add(a1);
        artifacts.add(a2);
        artifacts.add(a3);
        Map<String, Collection<Artifact>> map = new HashMap<String, Collection<Artifact>>();
        map.put("not used key", artifacts);

        mojo.getVersionsFromArtifactItems(map, props);
        mojo.getVersionsFromProjectDependencies(props);
        mojo.getVersionsFromJoinedGroupIds(props);

        assertEquals("Only 3 properties should be included", 3, props.size());
        assertEquals("Property value is wrong", "3.0", props.get("org.ow2.util.test.filter"));
        assertEquals("Property value is wrong", "3.1", props.get("org.ow2.util.test.filter_test-unmatched-version"));
        assertEquals("Property value is wrong", "2.0", props.get("org.apache.maven_maven-core"));
    }

    public void testMojoExecutionSorting() throws Exception {

        VersionsMojo mojo = lookupVersionsMojo();

        Map<String, String> props = new HashMap<String, String>();

        mojo.storeVersionsProperties(props);
    }

    public void testMojoCreationPattern() throws Exception {

        VersionsMojo mojo = lookupVersionsMojo();

        Pattern p1 = mojo.createPattern("org.ow2.util.test");
        assertTrue("invalid pattern", p1.matcher("org.ow2.util.test").matches());
        assertFalse("invalid pattern", p1.matcher("org.ow2.util.test.fail").matches());
        assertFalse("invalid pattern", p1.matcher("orgXow2XutilXtest").matches());

        Pattern p2 = mojo.createPattern("org.ow2.util.test.*");
        assertFalse("invalid pattern", p2.matcher("org.ow2.util.test").matches());
        assertTrue("invalid pattern", p2.matcher("org.ow2.util.test.fail").matches());
        assertTrue("invalid pattern", p2.matcher("org.ow2.util.test.*").matches());
        assertFalse("invalid pattern", p2.matcher("orgXow2XutilXtest").matches());

        Pattern p3 = mojo.createPattern("org.ow2.util.test*");
        assertTrue("invalid pattern", p3.matcher("org.ow2.util.test").matches());
        assertTrue("invalid pattern", p3.matcher("org.ow2.util.test.fail").matches());
        assertFalse("invalid pattern", p3.matcher("orgXow2XutilXtest").matches());
    }

    private VersionsMojo lookupVersionsMojo() throws Exception {
        File testPom = new File(getBasedir(),
                                "src/test/resources/org/ow2/util/maven/jbuilding/test-versions-mojo.pom" );
        VersionsMojo mojo = (VersionsMojo) lookupMojo("versions", testPom);
        assertNotNull(mojo);
        return mojo;
    }

    private VersionsMojo lookupConfiguredVersionsMojo() throws Exception {
        File testPom = new File(getBasedir(),
                                "src/test/resources/org/ow2/util/maven/jbuilding/test-file-creation-versions-mojo.pom" );

        when(repositorySystem.createArtifactWithClassifier(anyString(), anyString(), anyString(), anyString(), anyString()))
                .thenReturn(mavenCoreArtifact);
        when(repositorySystem.resolve(any(ArtifactResolutionRequest.class)))
                .thenReturn(mavenCoreResult);
        when(mavenCoreResult.isSuccess())
                .thenReturn(Boolean.TRUE);
        when(mavenCoreResult.getArtifactResolutionNodes())
                .thenReturn(Collections.singleton(new ResolutionNode(mavenCoreArtifact, null)));

        getContainer().addComponent(repositorySystem, RepositorySystem.class, null);

        VersionsMojo mojo = (VersionsMojo) lookupConfiguredMojo(
                new TestableMavenProject(testPom),
                "versions"
        );

        assertNotNull(mojo);
        return mojo;
    }

    private Artifact createArtifact(String gId, String aId, String version) {
        Artifact artifact = new ArtifactStub();
        artifact.setGroupId(gId);
        artifact.setArtifactId(aId);
        artifact.setVersion(version);
        return artifact;
    }
}
